
/*=============================================================================
   beest, an HTML viewer.
   Copyright (C) 2000 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at zeyr@excite.com.
=============================================================================*/

/*----------------------------------------------------------------------
 * Module name:    color.c
 * Author name:    Zach Smith
 * Create date:    29 Nov 2000
 * Last change:    4 Dec 2000
 * Purpose:        Translation of color information.
 *----------------------------------------------------------------------
 * Changes:
 * 4 Dec 2000, zeyr@excite.com: Added conversion of #RRGGBB to Color type.
 *--------------------------------------------------------------------*/


#include <stdio.h>
#include <ctype.h>

#include "defs.h"
#include "main.h"
#include "error.h"
#include "color.h"


typedef struct {
	char *name;
	Color color;
}
PresetColorEntry;


/* Yes, yet another prearranged hash to speed things up.
 */
static PresetColorEntry preset_colors_a [] = { 
{"aquamarine", 0x60d0b0},
{ NULL, 0 }
};

static PresetColorEntry preset_colors_b [] = { 
{"black", 0x0},
{"blue", 0x0000ff},
{"brown", 0xff4040},
{ NULL, 0 }
};

static PresetColorEntry preset_colors_c [] = { 
{"cyan", 0x00ffff},
{ NULL, 0 }
};

static PresetColorEntry preset_colors_g [] = { 
{"green", 0x00ff00},
{"gold", 0xf0d000},

{"gray", 0xb8b8b8}, /* same as gray6 */
{"grey", 0xb8b8b8},

{"gray0", 0x0},
{"grey0", 0x0},
{"gray1", 0x404040},
{"grey1", 0x404040},
{"gray2", 0x585858},
{"grey2", 0x585858},
{"gray3", 0x707070},
{"grey3", 0x707070},
{"gray4", 0x888888},
{"grey4", 0x888888},
{"gray5", 0xa0a0a0},
{"grey5", 0xa0a0a0},
{"gray6", 0xb8b8b8},
{"grey6", 0xb8b8b8},
{"gray7", 0xd0d0d0},
{"grey7", 0xd0d0d0},
{"gray8", 0xe8e8e8},
{"grey8", 0xe8e8e8},
{"gray9", 0xffffff},
{"grey9", 0xffffff},

{ NULL, 0 }
};

static PresetColorEntry preset_colors_m [] = { 
{"magenta", 0xff00ff},
{"maroon", 0xb03060},
{ NULL, 0 }
};

static PresetColorEntry preset_colors_n [] = { 
{"navyblue", 0x000080},
{ NULL, 0 }
};

static PresetColorEntry preset_colors_o [] = { 
{"orange", 0xffa500},
{ NULL, 0 }
};

static PresetColorEntry preset_colors_p [] = { 
{"purple", 0xa020f0},
{ NULL, 0 }
};

static PresetColorEntry preset_colors_r [] = { 
{"red", 0xff0000},
{ NULL, 0 }
};

static PresetColorEntry preset_colors_s [] = { 
{"sienna", 0xa05030},
{ NULL, 0 }
};

static PresetColorEntry preset_colors_t [] = { 
{"turquoise", 0x40e0d0},
{ NULL, 0 }
};

static PresetColorEntry preset_colors_v [] = { 
{"violet", 0xf080f0},
{ NULL, 0 }
};

static PresetColorEntry preset_colors_w [] = { 
{"white", 0xffffff},
{ NULL, 0 }
};

static PresetColorEntry preset_colors_y [] = { 
{"yellow", 0xffff00},
{ NULL, 0 }
};


static PresetColorEntry* preset_colors [26] = { 
	preset_colors_a, preset_colors_b, preset_colors_c, NULL,
	NULL, NULL, preset_colors_g, NULL,
	NULL, NULL, NULL, NULL,
	preset_colors_m, preset_colors_n, preset_colors_o, preset_colors_p,
	NULL, preset_colors_r, preset_colors_s, preset_colors_t,
	NULL, preset_colors_v, preset_colors_w, NULL,
	preset_colors_y, NULL,
};


/* Routine to translate a name or #RRGGBB expression to a
 * 24-bit RGB value.
 *
 * This routine has nothing to do with the SVGA color table.
 * Later this routine will make use of information such as 
 * that found in X-Window's rgb.txt.
 */
Color
color_lookup (String name)
{
	Color result = INVALID_COLOR;
	PresetColorEntry *pce;
	int i;

	CHECK_PARAM_NOT_NULL(name);

	/* Process colors that are specified with hex notation */
	if ('#'==*name) {
		name++;
		if (strlen(name)==6) {
			result=0;
			result += h2toi (name);
			result <<= 8;
			name+=2;
			result += h2toi (name);
			result <<= 8;
			name+=2;
			result += h2toi (name);
		}
		else 
		if (strlen(name)==3) {
			result=0;
			result += (16 * h2toi (name));
			name++;
			result <<= 8;
			result += (16 * h2toi (name));
			name++;
			result <<= 8;
			result += (16 * h2toi (name));
		}
		return result;
	}

	i = tolower(*name) - 'a';

	/* Reject colors that don't begin with a letter */
	if (i<0 || i>=26) 
		return result;

	pce = preset_colors [i];
	if (pce) {
		char *ptr;
		i=0;
		while ((ptr = pce->name)) {
			if (my_case_insensitive_strcmp (ptr,name)) {
				result = pce->color;
				break;
			}
			pce++;
		}
	}

	if (result==INVALID_COLOR && debug_mode && logfile) {
		fprintf (logfile, "COLOR_LOOKUP: failed to translate %s\n",name);
	}

	return result;
}

