
/*=============================================================================
   beest, an HTML viewer.
   Copyright (C) 2000 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at zeyr@excite.com.
=============================================================================*/

/*----------------------------------------------------------------------
 * Module name:    interact.c
 * Author name:    Zach Smith
 * Create date:    11 Dec 2000
 * Last change:    11 Dec 2000
 * Purpose:        Contains interaction() routine
 *----------------------------------------------------------------------
 * Changes:
 *--------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

#ifdef TURBO_C
#include <conio.h>
#include <alloc.h>
#endif

#include "defs.h"
#include "display.h"
#include "main.h"
#include "parse.h"
#include "word.h"
#include "group.h"
#include "error.h"
#include "svga.h"
#include "malloc.h"



/*-----------------------------------------------------------------------*/


void
draw_title_and_copyright (DisplayDevice *dpy, Coordinate x, Coordinate y,
	Dimension h)
{
	char str[200];

	/* Draw the title/copyright text */
	sprintf (str, "This is Beest, version %s", PROGRAM_VERSION);
	dpy->draw_text (x,y,str,0);
	y+=h;
	dpy->draw_text (x,y,"Copyright (C) 2000 by Zach Smith",0);
	y+=h;
	dpy->draw_text (x,y,"This program is provided AS IS, without any warranty whatsoever,",0);
	y+=h;
	dpy->draw_text (x,y,"and is covered by the GNU Public License. For more information,",0);
	y+=h;
	dpy->draw_text (x,y,"see the included file COPYING.",0);
	y+=h;
	dpy->draw_text (x,y,"The author is reachable at zeyr@excite.com.",0);
	y+=h;
	dpy->draw_text (x,y,"Also note:",0);
	y+=h;
	dpy->draw_text (x,y,"Compiled-in fonts (Times, Helvetica, Courier, and NewCenturySchoolbook) are",0);
	y+=h;
	dpy->draw_text (x,y,"Copyright (C) 1984, 1987 Adobe Systems Incorporated. All rights reserved.",0);
	y+=h;
	dpy->draw_text (x,y,"Copyright (C) 1988, 1991 Digital Equipment Corporation. All Rights Reserved.",0);
}



/* Routine to display banner from an XBM file.
 * I'm not using it since I decided to draw the Beest name with lines.
 */
void
draw_banner (DisplayDevice *dpy)
{
	FILE *bannerfile;
	Coordinate x,y;
	Dimension h;
	Image im;

	dpy->reset_colors();

	bannerfile = fopen ("banner.xbm", "r");
	if (!bannerfile) return;

	im.pixmap = NULL;
	if (!image_read_xbm (&im, bannerfile, dpy->fg, dpy->bg))
		return;

	dpy->set_font_complete ("Helvetica",14,0);
	h = dpy->get_font_height();
	if (!h) error_handler ("SVGA: style set but no font data");

	{
		Color c = color_lookup ("cyan");
		dpy->set_colors (0xff0000,c);
	}

	x = (dpy->actual_width - im.width)/2;
	y = (dpy->height - im.height - 10*h)/2;
	if (y<10) y=10;
	dpy->put_image (x, y, &im);
	my_free (im.pixmap);

	y += im.height + 5;

	draw_title_and_copyright (dpy, x, y, h);

	dpy->reset_colors();
}


void
draw_logo (DisplayDevice *dpy)
{
	Coordinate x,y;
	int w, n, s, i;
	int left;
	n = 20;
	w = 110;
	s = 120;

	/* draw_banner (dpy); */

	left = (dpy->actual_width - 5*s) / 2;

	for (i=0; i<8; i++) {
		Color color;
		char *colorname;

		x=left+i/2;
		y=150+i-i/3;

		switch (i) {
		case 0: colorname="grey1"; break;
		case 1: colorname="grey1"; break;
		case 2: colorname="grey2"; break;
		case 3: colorname="grey2"; break;
		case 4: colorname="grey3"; break;
		case 5: colorname="grey4"; break;
		case 6: colorname="grey6"; break;
		case 7: colorname="grey8"; break;
		}
		color = color_lookup (colorname);
		dpy->set_colors (color,0);

		dpy->draw_line (x,y+n,x+w,y); y+=n;
		dpy->draw_line (x,y+n,x+w,y); y+=n;
		dpy->draw_line (x,y+n,x+w,y); y+=n;
		y-=3*n;
		dpy->draw_line (x+w/4,y+n-n/4,x+w/4,y+3*n);
		dpy->draw_line (x+w,y,x+w-w/4,y+n+n/4);
		dpy->draw_line (x+w,y+n,x+w-w/5,y+2*n+n/5);
		/*dpy->draw_line (x+w-w/5,y+n+n/5,x+w-w/5,y+2*n+n/5);*/
		x+=s;

		dpy->draw_line (x,y+n,x+w,y); y+=n;
		dpy->draw_line (x,y+n,x+w,y); y+=n;
		dpy->draw_line (x,y+n,x+w,y); y+=n;
		y-=3*n;
		dpy->draw_line (x+w/4,y+n-n/4,x+w/4,y+3*n);
		x+=s;

		dpy->draw_line (x,y+n,x+w,y); y+=n;
		dpy->draw_line (x,y+n,x+w,y); y+=n;
		dpy->draw_line (x,y+n,x+w,y); y+=n;
		y-=3*n;
		dpy->draw_line (x+w/4,y+n-n/4,x+w/4,y+3*n);
		x+=s;

		dpy->draw_line (x,y+n,x+w,y); y+=n;
		dpy->draw_line (x,y+n,x+w,y); y+=n;
		dpy->draw_line (x,y+n,x+w,y); y+=n;
		y-=3*n;
		dpy->draw_line (x+w/4,y+n-n/4,x+w/4,y+2*n);
		dpy->draw_line (x+w-w/4,y+n+n/4,x+w-w/4,y+2*n+n/4);
		x+=s;

		dpy->draw_line (x,y+n,x+w,y); y+=n;
		dpy->draw_line (x+w-6*w/10,y-n/3,x+w-6*w/10,y+n*2-n/3);
		dpy->draw_line (x+w-4*w/10,y-2*n/3,x+w-4*w/10,y+n*3);
		y-=3*n;
		x+=s;
	}

	dpy->reset_colors();
}



/* Quotations
*/
static String quotations[] = {
	"Felix qui potuit rerum cognoscere causas.\n(Happy is he who knows the causes of things.)",
	"The price of freedom is eternal vigilance. \n-Thomas Jefferson",
	"Those who would deny freedom to others deserve it not for themselves.\n-Abraham Lincoln",
	"Parvum parva decent\nRough translation... Always test your limits",
	"A penny saved is a penny earned.\n-Ben Franklin",
};



void
undraw_quotation (DisplayDevice *dpy, Image *im) {
	if (im) {
		dpy->pointer_hide ();
		dpy->put_image (im->x,im->y,im);
		dpy->pointer_unhide ();
	}
}

void
draw_quotation (DisplayDevice *dpy, Image *im)
{
	int which = time(NULL) & 7;
	char string1[200];
	char string2[200];
	char *tmp;

	if (which > 4) which /= 2;

	strcpy (string1, quotations[which]);
	if (!strlen (string1)) return;

	dpy->reset_colors();
	dpy->set_font_complete ("Times",14,0);

	tmp = strchr (string1, '\n');
	if (tmp) {
		*tmp = 0;
		strcpy (string2, "      ");
		strcat (string2, tmp+1);
	}
	else
		string2[0]=0;

	dpy->draw_twoline_centered_popup (string1,string2,im);
}


void
interaction (DisplayDevice *dpy, Group *toplevel)
{
	DisplayEvent de;
	Coordinate ix, iy;
	int scroll_amount;
	int page_scroll_amount;
	int lastline; /* last -screen- line */
	time_t t0;	
	time_t t;	
	Image *im=NULL;
	unsigned char quotation_is_displayed=!noquote_mode;
	Dimension width, height;

	CHECK_PARAM_NOT_NULL(dpy);
	CHECK_PARAM_NOT_NULL(toplevel);

	width = toplevel->width;
	height = toplevel->height;

	if (toplevel->height < dpy->height) {
		dpy->vscroll_disable();
	}

	if (toplevel->width < dpy->width) {
		dpy->hscroll_disable();
		if (dpy->vscroll_width) {
			dpy->vscroll_disable();
			dpy->vscroll_enable();
		}
	}

	scroll_amount = dpy->height / 6;
	page_scroll_amount = dpy->height / 2;

	ix = 0;
	iy = 0;

	/* Wait until the last minute to clear away the banner.
	 */
	dpy->clear();

	/* Draw a full screen of the text
	 */
	t0 = time(NULL);
	group_redraw (dpy, toplevel, ix, iy, 0, 0, dpy->width-1, dpy->height-1);
	t = time(NULL);
	if (logfile) fprintf (logfile, "seconds to redraw screen: %lu\n", t-t0);

	/* On top of the text, draw the quotation
	 */
	if (!noquote_mode) {
		im = image_new();
		draw_quotation (dpy, im);
		t0 = time(NULL);
	}

	lastline=dpy->height-1;
	height -= dpy->height + dpy->hscroll_height;

printf ("dpy->height = %d\n", (short) dpy->height);

	ix=iy=0;

	while (TRUE) {
		int ch;

		while (!dpy->get_event (&de)) {

			if (!noquote_mode) {
				t = time(NULL);
				if (quotation_is_displayed && t>=(t0+4)) {
					undraw_quotation (dpy,im);
					quotation_is_displayed = FALSE;
				}
				if (quotation_is_displayed) 
					continue;
			}
		}

		/* Any key or mouse event should erase
		 * the quotation.
		 */
		if (!noquote_mode && quotation_is_displayed) {
			undraw_quotation (dpy,im);
			quotation_is_displayed = FALSE;
		}

		switch (de.type) {
		case EVENT_MOUSE:
			if (logfile) 
			    fprintf (logfile,"mouse event: (%ld,%ld) buttons: %d\n",
				de.x0,de.y0,de.value);

			if (de.value) {
				SelectionPoint sp;

				if (group_identify_target(
						toplevel,
					     	de.x0+ix,
						de.y0+iy,
						&sp))
				{
					dpy->reset_colors();
					dpy->pointer_hide();
					dpy->highlight_area (
						sp.ax-ix,
						sp.ly0-iy,
						sp.ax-ix+sp.word->width-1,
						sp.ly1);
					dpy->pointer_unhide();
fprintf (logfile,"FOUND: w->str=%s\n", sp.word->str);
				}
			}

#if 0
			if (de.value & MOUSE_LEFTBUTTON)
			printf ("left button pressed\n");
			if (de.value & MOUSE_RIGHTBUTTON)
			printf ("right button pressed\n");
#endif

			continue;
		case EVENT_UP:
			if (logfile) fprintf (logfile,"key event: Up key\n");
			if (!dpy->vscroll_width) continue;

			if (iy>scroll_amount) { 
				long tmp;

				dpy->scroll_down (scroll_amount);
				iy -= scroll_amount;
				dpy->restore_background (0, scroll_amount);
				group_redraw (dpy,toplevel,ix,iy,0,0,
					dpy->width-1,scroll_amount);
				tmp = 100 * iy;
				tmp /= height;
				dpy->vscroll_move(tmp);
			} 
			else if (iy) 
			{
				dpy->scroll_down(iy);
				dpy->restore_background (0, iy);
				group_redraw (dpy,toplevel,ix,0,0,0,dpy->width-1,
					iy);
				iy = 0;
				dpy->vscroll_move(0);
			}
			break;
		case EVENT_PGUP:
			if (logfile) fprintf (logfile,"key event: Page Up key\n");
			if (!dpy->vscroll_width) continue;

			if (iy>page_scroll_amount) { 
				long tmp;

				dpy->scroll_down (page_scroll_amount);
				iy -= page_scroll_amount;
				dpy->restore_background (0, page_scroll_amount);
				group_redraw (dpy,toplevel,ix,iy,0,0,
					dpy->width-1,page_scroll_amount);
				tmp = 100 * iy;
				tmp /= height;
				dpy->vscroll_move(tmp);
			} 
			else if (iy) 
			{
				dpy->scroll_down(iy);
				dpy->restore_background (0, iy);
				group_redraw (dpy,toplevel,ix,0,0,0,
					dpy->width-1,iy);
				iy = 0;
				dpy->vscroll_move(0);
			}
			break;
		case EVENT_DOWN:
			if (logfile) fprintf (logfile,"key event: Down key\n");
			if (!dpy->vscroll_width) continue;

			if (iy < height) {
				if ((iy+scroll_amount) < (height-scroll_amount)) { 
				  dpy->scroll_up(scroll_amount);
				  dpy->restore_background (lastline-scroll_amount,lastline);
				  iy += scroll_amount;
				  group_redraw (dpy,toplevel,ix,iy,0,lastline-scroll_amount,dpy->width,lastline);
					
				  dpy->vscroll_move((100*iy)/height);
				} 
				else
				{
				  dpy->scroll_up (height-iy);
				  dpy->restore_background (lastline-iy,lastline);
				  iy = height;
				  group_redraw (dpy,toplevel,ix,iy,0,lastline-iy,dpy->width,lastline);
				  dpy->vscroll_move(100);
				}
			}
			break;
		case EVENT_PGDN:
			if (logfile) fprintf (logfile,"key event: Page Down key\n");
			if (!dpy->vscroll_width) continue;

			if (iy < height) {
				long tmp;
				if ((iy+page_scroll_amount) < (height-page_scroll_amount)) { 
				  dpy->scroll_up(page_scroll_amount);
printf ("restoring from %d to %d\n", lastline-page_scroll_amount, lastline);
				  dpy->restore_background (lastline-page_scroll_amount,lastline);
				  iy += page_scroll_amount;
				  group_redraw (dpy,toplevel,ix,iy,0,lastline-page_scroll_amount,dpy->width,lastline);
				  tmp = 100 * (long) iy;
				  tmp /= (long) height;
				  dpy->vscroll_move(tmp);
				} 
				else
				{
				  int diff = height-iy;
				  dpy->scroll_up (diff);
				  dpy->restore_background (lastline-diff,lastline);
				  iy = height;
				  group_redraw (dpy,toplevel,ix,iy,0,lastline-diff,dpy->width-1,lastline);
				  dpy->vscroll_move(100);
				}
			}
			break;
		case EVENT_HOME:
			if (logfile) fprintf (logfile,"key event: Home key\n");
			ix = iy = 0;
			dpy->restore_background (0,lastline);
			group_redraw (dpy, toplevel, ix, iy, 0, 0, dpy->width-1, dpy->height-1);
			dpy->vscroll_move(0);
			break;
		case EVENT_END:
			if (logfile) fprintf (logfile,"key event: End key\n");
			dpy->restore_background (0,lastline);
			ix = 0;
			iy = toplevel->height - dpy->height;
			group_redraw (dpy,toplevel,ix,iy,0,0,dpy->width,lastline);
			dpy->vscroll_move(100);
			break;
		case EVENT_LEFT:
			if (logfile) fprintf (logfile,"key event: Left key\n");
			break;
		case EVENT_RIGHT:
			if (logfile) fprintf (logfile,"key event: Right key\n");
			break;
		case EVENT_KEY:
			if (logfile) 
			    fprintf (logfile,"key event: (code = %d)\n",
				de.value);
			ch = de.value;
			switch (ch) {
			case 'q': return;
			case 'w': {
				FILE *f = fopen("out.htm","w");
				fprintf (logfile, "Writing duplicate HTML file...\n");
				group_write_html(f,dpy,toplevel);
				fclose(f);
				break;
			}
			case 's': {
				Image im;
				FILE *f = fopen("screen.xbm","w");
				fprintf (logfile,"Writing screen dump as XBM file...\n");
				dpy->get_image (0,0,640,480,&im);
				image_write_xbm (&im, f);
				fclose(f);
				break;
			}
			}
		default:
			fprintf (logfile, 
			    "unknown or non-implemented event type %d\n",
				de.type);
			continue;
		}

	}
}



