
/*=============================================================================
   beest, an HTML viewer.
   Copyright (C) 2000 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at zeyr@excite.com.
=============================================================================*/

/*----------------------------------------------------------------------
 * Module name:    line.c
 * Author name:    Zach Smith
 * Create date:    1 Oct 2000
 * Last change:    26 Nov 2000
 * Purpose:        Line class, corresponding to a screen line of objects.
 *----------------------------------------------------------------------
 * Changes:
 *--------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>

#include "defs.h"
#include "main.h"
#include "malloc.h"
#include "word.h"
#include "error.h"
#include "line.h"


 
Line*
line_new () {
	Line *line;

	line = (Line*) my_malloc (sizeof(Line));
	if (!line) error_handler ("unable to allocate Line object");

	memset (line, 0, sizeof (Line));
	return line;
}


void
line_free (Line *l) {
	Line *next;

	CHECK_PARAM_NOT_NULL(l);

	while (l) {
		next = l->next;
		my_free((char*) l);
		l = next;
	}
}


void
line_add_word (Line *l, Word *w) {
	CHECK_PARAM_NOT_NULL(l);
	CHECK_PARAM_NOT_NULL(w);

	if (!l->word_list) {
		l->word_list = l->last_word = w;
		w->next_in_line=NULL;
	} else {
		l->last_word->next_in_line = w;
		l->last_word = w;
	}
	++l->total_words;

/* printf ("in line_add_word, w->ascent=%u\n", w->ascent); */

#if 0
	/* Note! non-displayed objects and functional objects (<hr> etc)
	 * may have zero or negative dims.
	 */
	if (w->width > 0 && (w->ascent+w->descent) > 0) {
#endif
		l->width += w->width;
		if (l->ascent < w->ascent)
			l->ascent = w->ascent;
		if (l->descent < w->descent)
			l->descent = w->descent;
#if 0
	} else {
printf ("dimensionless obj %s \n",w->str);
	}
#endif
}


void
line_dump (FILE *f, Line *l) {
	Word *w;

	CHECK_PARAM_NOT_NULL(l);
	
	fprintf (f, "LINE DUMP, total=%d ascent=%lu descent=%lu \n", 
		l->total_words, l->ascent, l->descent);
	w = l->word_list;
	while (w) {
		fprintf (f, " word in list: '%s' at %ld,%ld (%lu x [%lu + %lu]), ", w->str,
			w->x, w->y, w->width, 
			w->ascent,
			w->descent);
		style_dump (f, w->style);
		fprintf (f, "\n");
		w = w->next_in_line;
	}
	fprintf (f,"END OF LINE DUMP\n");
}


void 
linelist_dump (FILE *f, Line *l) {
	int num=0;

	CHECK_PARAM_NOT_NULL(l);

	while (l) {
		fprintf (f,"Line %d is at y coordinate %ld and has ascent %lu descent %lu \n",
			num++, 
			l->y, 
			l->ascent,
			l->descent);
		line_dump(f,l);
		l=l->next;
	}
}


