
/*=============================================================================
   beest, an HTML viewer.
   Copyright (C) 2000 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at zeyr@excite.com.
=============================================================================*/

/*----------------------------------------------------------------------
 * Module name:    malloc.c
 * Author name:    Zach Smith
 * Create date:    1 Oct 2000
 * Last change:    26 Nov 2000
 * Purpose:        Routines for monitoring memory use.
 *----------------------------------------------------------------------
 * Changes:
 * 5 Dec 2000, zeyr@excite.com: strict tracing is now optional
 *--------------------------------------------------------------------*/


#include <stdio.h>


#ifdef TURBO_C
#include <alloc.h>
#else
#include <malloc.h>
#endif


#include "defs.h"
#include "error.h"
#include "main.h"


static unsigned long total_allocated=0;


#if 0
#define STRICT_MALLOC_TRACING
#endif


#ifdef STRICT_MALLOC_TRACING
static unsigned long total_freed=0;
static char * malloc_log [200000];
static unsigned long malloc_size[200000];
static unsigned long last_malloc=0;
#endif


char *
my_malloc (unsigned long amount)
{
	char *ptr;
	ptr = malloc (amount);
#ifdef TURBO_C
	if (!ptr) {
		fprintf (stderr,"Available memory: %lu, request size: %lu\n",
			farcoreleft(), amount);
	}
#endif
	total_allocated += amount;

#ifdef STRICT_MALLOC_TRACING
	if (last_malloc==200000)
		error_handler ("too many mallocs for debugging");

	malloc_log[last_malloc] = ptr;
	malloc_size[last_malloc++] = amount;
#endif

	return ptr;
}


void
my_free (char *ptr)
{
#ifdef STRICT_MALLOC_TRACING
	unsigned long i=0;
	while (i<last_malloc && malloc_log[i] != ptr)
		i++;
	if (i==last_malloc)
		error_handler ("attempt to free pointer that was never malloc'd");

	total_freed += malloc_size[i];
#endif

	// free (ptr); 
}


void
my_malloc_total (unsigned long *alloced, unsigned long *freed) {
	*alloced = total_allocated;
#ifdef STRICT_MALLOC_TRACING
	*freed = total_freed;
#else
	*freed = 0;
#endif
}



String
my_strdup (String str)
{
	String s2;
	unsigned long len;

	CHECK_PARAM_NOT_NULL(str);

	len=strlen(str)+1;
	s2 = (String) my_malloc (len);
	memcpy (s2,str,len);
	
	return s2;
}


