
/*=============================================================================
   beest, an HTML viewer.
   Copyright (C) 2000 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at zeyr@excite.com.
=============================================================================*/

/*----------------------------------------------------------------------
 * Module name:    word.h
 * Author name:    Zach Smith
 * Create date:    1 Oct 2000
 * Last change:    26 Nov 2000
 * Purpose:        Type definitions, prototypes, defines for Word and
 *                 Segment classes.
 *----------------------------------------------------------------------
 * Changes:
 * 1 Dec 2000, zeyr@excite.com: addition of tag number
 *--------------------------------------------------------------------*/

#ifndef _WORD
#define _WORD

#ifndef _DISPLAY
#include "display.h"
#endif

#ifndef _STYLE
#include "style.h"
#endif

typedef struct seg {
	String str;
	struct seg *next;
}
Segment;


typedef struct word 
{
	String str;

	struct word *next;
	struct word *child;
	struct word *next_in_line;

	/* Subgroup: any tag which creates a subgroup needs to
	 * point to it so that later we can reproduce HTML during
	 * the word processor's File-Save operation.
	 */
	void *subgroup;

	Style *style; /* NULL if no change from previous word on Line */

	Segment *segments;
	Segment *last_segment;
	unsigned char total_segments;

	/* This is set to a value >0 whenever a Word is followed by
	 * whitespace, and things are done this way in order to
	 * save precious memory on more limited systems. Otherwise,
	 * a Word which contains only the " " string would take up about
	 * fifty bytes. The parser sets this to a value of 1, but
	 * group_geometry sets it to the on-screen width.
	 */
	unsigned short flags;
	unsigned int para_align : 2;
#define WORD_HAS_FOLLOWING_WHITESPACE (1)
#define WORD_HAS_PRECEDING_PARAGRAPH_BREAK (2)
#define WORD_HAS_PRECEDING_LINE_BREAK (4)
#define WORD_HAS_PRECEDING_HEADING1 (8)
#define WORD_HAS_PRECEDING_HEADING2 (16)
#define WORD_HAS_PRECEDING_HEADING3 (32)
#define WORD_HAS_PRECEDING_HEADING4 (64)
#define WORD_HAS_PRECEDING_HEADING5 (128)
#define WORD_HAS_PRECEDING_HEADING6 (256)
#define WORD_HAS_PRECEDING_HEADING_END (512)

	/* 0=none, 1=<P>, 2=<BR> etc...
	 */
	unsigned tag_number : 7;

	/* Utility datum and pointer for use by tags.
	 * For <table> this is a ptr to a Table object.
	 * For <hr> this is the clip width (unsigned long).
	 */
	union {
		void *ptr; 
		void *table;
		unsigned long datum;
	} tag;

	/* Assigned for later redraws */
	Coordinate x, y; 
	Dimension width;
	Dimension min_width; /* same as width unless object has children */
	Dimension ascent;
	Dimension descent;
}
Word;


extern Word* word_new (String);
extern void word_free (Word*);
extern void word_dump(FILE*,Word *);
extern void word_redraw (DisplayDevice*, Word *);
extern int word_divide_tag (Word*);
extern String word_fetch_tag_var_value(Word*,String);
extern void word_apply_offset (Word*,Coordinate,Coordinate);

#endif
